<?php
namespace Scriptor\Modules\PagesMetatags;

use Imanager\Field;
use Imanager\TemplateParser;
use Scriptor\Core\Module;
use Scriptor\Core\Scriptor;
use Scriptor\Core\Site;

class PagesMetatags extends Module {

    public Site $site;

    private TemplateParser $templateParser;

    const FIELD_NAME = 'meta';

    const FIELD_KEYS = [
        'title' => '',
        'description' => '',
        'keywords' => ''
    ];

    public static function moduleInfo() : array
    {
        return [
            'name' => 'PagesMetatags',
            'version' => '1.0.0',
            'description' => 'The PagesMetatags Module is a Scriptor editor extension for storing and managing page metadata.'.
                ' Optimize your pages for search engines and improve visibility by easily adding information like titles, '.
                'descriptions and keywords.',
            'author' => 'Your Name',
            'author_website' => 'https://your-website.com'
        ];
    }

    public static function moduleHooks() : array
    {
        return [
            'Pages::afterRenderEditorTemplateField' => [
                [
                    'module' => 'PagesMetatags',
                    'method' => 'renderPageMetaFields'
                ]
            ],
            'Pages::afterSavePage' => [
                [
                    'module' => 'PagesMetatags',
                    'method' => 'savePageMeta'
                ]
            ]
        ];
    }

    public function init()
    {
        parent::init();
        $this->site = Scriptor::getSite();
        $this->templateParser = new TemplateParser();
    }

   /**
    * Generates and displays markup of meta fields in the page editor.
    * 
    * @param object $event - Hook event
    */
    public function renderPageMetaFields($event)
    {
        // Get current page
        $page = $event->object->page;

        // Embed custom style sheet
        $this->addResource('link', [
            'rel' => 'stylesheet', 
            'href' => dirname($this->siteUrl).'/site/modules/PagesMetatags/css/styles.css'
        ], 'header');
        
        // Use metafield data of the current page, if available 
        $meta = isset($page->meta) ? $page->meta : self::FIELD_KEYS;    
    
        // Build meta fields markup
        ob_start(); ?>
        <div class="pmt form-control">
            <fieldset>
                <legend><?php echo $this->i18n['metadata_legend']; ?></legend>
                <div class="form-control">
                    <label for="metatitle"><?php echo $this->i18n['metatitle_label']; ?></label>
                    <p class="info-text i-wrapp"><i class="gg-danger"></i> <?php
                        echo $this->i18n['metatitle_field_infotext'] ?></p>
                    <input name="metatitle" id="metatitle" type="text" value="<?php 
                       echo $meta['title']; ?>">
                </div>
                <div class="form-control">
                    <label for="metadescription"><?php 
                      echo $this->i18n['metadescription_label']; ?></label>
                    <p class="info-text i-wrapp"><i class="gg-danger"></i> <?php
                        echo $this->i18n['metadescription_field_infotext'] ?></p>
                    <input name="metadescription" id="metadescription" type="text" 
                        value="<?php echo $meta['description']; ?>">
                </div>
                <div class="form-control">
                    <label for="metakeywords"><?php 
                      echo $this->i18n['metakeywords_label']; ?></label>
                    <p class="info-text i-wrapp"><i class="gg-danger"></i> <?php
                        echo $this->i18n['metakeywords_field_infotext'] ?></p>
                    <input name="metakeywords" id="metakeywords" type="text" value="<?php 
                        echo $meta['keywords']; ?>">
                </div>
            </fieldset>
        </div>
        <?php
        // Add the markup to the hooked methods return
        $event->return .= ob_get_clean();
    }

    /**
     * Save the metadata if the page was saved successfully.
     * 
     * @param object $event - Hook event
     * 
     */
    public function savePageMeta($event)
    {
        // Check if saving the page was successful, if not just leave
        if (!$event->return) return;
        $page = $event->object->page;
        // Prepare meta before saving
        $meta = [
            'title' => $this->sanitizer->text($this->input->post->metatitle),
            'description' => $this->sanitizer->text($this->input->post->metadescription),
            'keywords' => $this->sanitizer->text($this->input->post->metakeywords)
        ];
        // Set metadata and save the page
        $event->return = $page->set('meta', $meta)->save();
        // Display error message if saving did not succeed
        if (!$event->return) {
            $this->addMsg('error', $this->templateParser->render(
                $this->i18n['error_saving_meta'])
            );
        }
    }

    public function install() : bool
    {
        $this->init();

        // Get the Pages category object from the hooked class.
        $pages = $this->site->pages()->category;
        // Check the field name already exists
        $exists = $pages->getField('name='.self::FIELD_NAME);
        // If the meta field already exists, display the corresponding message.
        if ($exists) {
            $this->addMsg('error', $this->templateParser->render(
                $this->i18n['meta_field_exists_error'], [
                    'field_name' => self::FIELD_NAME
                ])
            );
            return false;
        }

        // If the meta field does not exist yet, try to install it
        try {
            $field = new Field($pages->id);

            $field->set('name', self::FIELD_NAME)
                ->set('type', 'array')
                ->set('label', 'Page Metadata')
                ->save();

            $this->addMsg('success', $this->templateParser->render(
                $this->i18n['meta_field_succesful_created'], [
                    'field_name' => self::FIELD_NAME
                ])
            );

            return true;

        } catch (\Exception $e) {
            $this->addMsg('error', $this->templateParser->render(
                $this->i18n['meta_field_install_error'], [
                    'error' => $e->getMessage()
                ])
            );
            return false;
        }
    }

    public function uninstall() : bool
    {
        $this->init();

        $pages = $this->site->pages()->category;
        $field = $pages->getField('name='.self::FIELD_NAME);

        if ($field) {
            if ($pages->remove($field)) {
                $this->addMsg('success', $this->templateParser->render(
                    $this->i18n['meta_field_succesful_uninstalled'], [
                        'field_name' => self::FIELD_NAME
                    ])
                );
                return true;
            }
        }

        $this->addMsg('error', $this->templateParser->render(
            $this->i18n['meta_field_not_exists_error'], [
                'field_name' => self::FIELD_NAME
            ])
        );
        return false;
    }
}
